/***************************************************************************
 *   Any complaints or wishes please mail to suslic-2012@rambler.ru        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "suslictk.h"

void SuslicTk::onFavRecScripts()
{
	QAction *actionScript = qobject_cast<QAction*> ( sender() );
	if ( actionScript )
	{
		QString scriptName = actionScript->text();
		qDebug() <<"script name "<<scriptName;
		int index = scriptNameL.indexOf ( scriptName );
		qDebug() <<"script index "<<index;
		if ( index >=0 )	onScript ( scriptTextL[index], scriptNameL[index] );
	}
}
void SuslicTk::onScriptDlg()
{
	scdlg = new ScriptDlg ( this, Qt::Dialog, &scriptNameL, &scriptTextL );
	connect ( scdlg, SIGNAL ( _onScript ( QString, QString ) ), this, SLOT ( onScript ( QString, QString ) ) );
	connect ( scdlg, SIGNAL ( finished ( int ) ), this, SLOT ( onScriptDlgClose ( int ) ) );
	scdlg->show();
}

void SuslicTk::onScriptDlgClose ( int result )
{
	qDebug() <<"ScriptDlg closed with code "<<result;
	Q_CHECK_PTR ( scdlg );
	disconnect ( scdlg, SIGNAL ( _onScript ( QString, QString ) ), this, SLOT ( onScript ( QString, QString ) ) );
	disconnect ( scdlg, SIGNAL ( finished ( int ) ), this, SLOT ( onScriptDlgClose ( int ) ) );
	delete scdlg;
}

void SuslicTk::onRecScriptListChanged()
{
	qDebug() <<"onRecScriptListChanged";
	qDebug() <<"recScriptList.count()="<<recScriptList.count();
//	recScriptList.removeDuplicates();
	if ( recScriptList.count() == 1 )
	{
		if ( scriptNameL.contains ( recScriptList[0] ) )
		{
			actionRecScript1->setText ( recScriptList[0] );
			actionRecScript1->setVisible ( true );
			qDebug() <<"actionRecScript1 is visible";
			actionRecScript2->setText ( "" );
			actionRecScript2->setVisible ( false );
		}
		else
		{
			recScriptList.clear();
			actionRecScript1->setVisible ( false );
			actionRecScript2->setVisible ( false );
		}
	}
	if ( recScriptList.count() >= 2 )
	{
		if ( scriptNameL.contains ( recScriptList[0] ) && scriptNameL.contains ( recScriptList[1] ) )
		{
			actionRecScript1->setText ( recScriptList[0] );
			actionRecScript1->setVisible ( true );
			actionRecScript2->setText ( recScriptList[1] );
			actionRecScript2->setVisible ( true );
		}
		else if ( !scriptNameL.contains ( recScriptList[0] ) )
		{
			recScriptList.removeAt ( 0 );
			actionRecScript1->setText ( recScriptList[0] );
			actionRecScript1->setVisible ( true );
			qDebug() <<"actionRecScript1 is visible";
			actionRecScript2->setText ( "" );
			actionRecScript2->setVisible ( false );
		}
		else
		{
			recScriptList.removeAt ( 1 );
			actionRecScript1->setText ( recScriptList[0] );
			actionRecScript1->setVisible ( true );
			qDebug() <<"actionRecScript1 is visible";
			actionRecScript2->setText ( "" );
			actionRecScript2->setVisible ( false );
		}
	}
}
void SuslicTk::onScript ( QString script, QString name )
{
	qDebug() <<"onScript()";
	if ( askdir )
	{
		if ( QMessageBox::question ( this, "Script execution", tr ( "Active directory is<br><b>%1</b>.<br>Selected script will be applied to files, contained in this directory or it's subdirectories (depending on the script). If you want to change directory - press \"No\", if not - press \"Yes\" to proceed" ).arg ( lastdir ), QMessageBox::No|QMessageBox::Yes ) == QMessageBox::No )
		{
			onChDir();
		}
	}
	if ( recScriptList.last() != name )
	{
		if ( recScriptList.count() >=2 )
		{
			recScriptList = QStringList() <<recScriptList.last() <<name;
		}
		else recScriptList<<name;
		onRecScriptListChanged();
	}
	QRegExp rx1 ( "(\\/)(\\*)(.*)(\\*)(\\/)" );
	rx1.setMinimal ( true );
	script = script.replace ( rx1, "" ).trimmed();
	QStringList scriptSplit = script.split ( "\n", QString::SkipEmptyParts );
	for ( int i=0;i<scriptSplit.count();i++ )
	{
		qDebug() <<"script:\n\t"<<scriptSplit[i];
		if ( scriptSplit[i].startsWith ( "txt2pdf" ) ) onTxtScript ( scriptSplit[i] );
		else if ( script.startsWith ( "burst" ) ) onBurstScript ( script );
		else if ( script.startsWith ( "encrypt" ) ) onEncryptScript ( script );
		else
		{
			QMessageBox::warning ( this, "Script", tr ( "Unknown script!" ) );
			qDebug() <<"unknown script";
		}
	}
}
void SuslicTk::onEncryptScript ( QString script )
{
	qDebug() <<"onEncryptScript";
	QStringList strL = script.split ( " ", QString::SkipEmptyParts );
	if ( strL.count() < 4 )
	{
		qDebug() <<"invalid encrypt script";
		return;
	}
	QString encrString;
	for ( int i=3;i<strL.count();i++ )
	{
		encrString+=strL[i];
		if ( i != strL.count()-1 ) encrString+=" ";
	}
	qDebug() <<"encrString="<<encrString;
	QStringList ifiles = QDir ( lastdir ).entryList ( QStringList() <<"*.pdf"<<"*.PDF", QDir::Files|QDir::NoDotAndDotDot, QDir::Name );
	QRegExp rx2 ( strL[1] );
	rx2.setPatternSyntax ( QRegExp::Wildcard );
	ifiles = ifiles.filter ( rx2 );
	if ( ifiles.count() == 0 )
	{
		QMessageBox::information ( this, "Encrypt Script", tr ( "No files to encrypt!" ) );
		return;
	}
	QString ofile, cifile;strL[2].replace ( QRegExp ( "^\\/" ), "" );
	if ( ifiles.count() > 1 )
	{
		prog_com = new QProgressDialog ( tr ( "Converting files" ), tr ( "Cancel" ), 0, ifiles.count() , this );
		prog_com->show();
		prog_com->setWindowModality ( Qt::WindowModal );
	}

	for ( int i = 0;i<ifiles.count();i++ )
	{
		if ( ifiles.count() > 1 )
		{
			if ( prog_com->wasCanceled() ) break;
			prog_com->setLabelText ( ifiles[i] );
		}

		ifiles[i] = lastdir+"/"+ifiles[i];
		qDebug() <<ifiles[i];
		bool chan;
		cifile = checkPath ( ifiles[i], "encr", &chan );

		QString dump = pdftkBin+" \""+cifile+"\" dump_data output -";
		proc = new QProcess();
		proc->start ( dump );
		addTextPEc ( dump );
		if ( !proc->waitForStarted() ) {qDebug() <<"Couldn't start";return;return;}
		if ( !proc->waitForFinished () ) {qDebug() <<"Not finished!";return;return;}
		qDebug() <<"Exit  Code: "<<proc->exitCode() ;
		qDebug() <<"Exit Status "<<proc->exitStatus() ;
		QString sErr = proc->readAllStandardError();
		addTextPE ( proc->readAllStandardOutput() + "<br>" + sErr );
		if ( !sErr.isEmpty() )	QMessageBox::information ( this, "Encrypt Script", tr ( "File %1 is already encrypted" ).arg ( ifiles[i] ) );
		else
		{
			ofile = lastdir+"/"+strL[2];
			ofile.replace ( "*", QFileInfo ( ifiles[i] ).baseName() );
			if ( !ofile.endsWith ( ".pdf", Qt::CaseInsensitive ) ) ofile+=".pdf";
			qDebug() << ofile;
			ofile = checkSaveFile ( ofile );
			if ( !ofile.isEmpty() )
			{
				if ( ifiles[i] != ofile )
				{
					if ( QFile ( ofile ).exists() )
					{
						if ( QMessageBox::question ( this, "Encrypt Script", tr ( "File<br>%1<br>exists. Overwrite?" ).arg ( ofile ), QMessageBox::Yes|QMessageBox::No ) == QMessageBox::No )
						{
							ofile = QFileDialog::getSaveFileName ( this, tr ( "Save as" ), QFileInfo ( ofile ).absolutePath() );
							ofile = checkSaveFile ( ofile );
						}
					}
					if ( !ofile.isEmpty() )
					{
						if ( !QFileInfo ( ofile ).absoluteDir().exists() )
						{
							if ( !QDir().mkpath ( QFileInfo ( ofile ).absolutePath() ) )
								QMessageBox::warning ( this, "Encrypt Script", tr ( "Failed to create directory.<br>%1 will be skipped" ).arg ( ifiles[i] ) );
							else onEncryptCore ( cifile, ofile, encrString );
						}
						else onEncryptCore ( cifile, ofile, encrString );
					}
					else qDebug() <<"ofile empty";
				}
				else	addTextTE ( tr ( "<font color=red>Cannot convert %1 to %2. Input and output names should differ!</font>" ).arg ( ifiles[i] ).arg ( ofile ) );
			}
			else qDebug() <<"canceled by user";
		}
		if ( ifiles.count() > 1 ) prog_com->setValue ( i+1 );
	}
	if ( ifiles.count() > 1 ) prog_com->setValue ( ifiles.count() );
}
void SuslicTk::onBurstScript ( QString script )
{
	qDebug() <<"onBurstScript";
	QStringList strL = script.split ( " ", QString::SkipEmptyParts );
	if ( strL.count() < 4 )
	{
		qDebug() <<"invalid burst script";
		return;
	}
	QStringList ifiles = QDir ( lastdir ).entryList ( QStringList() <<"*.pdf"<<"*.PDF", QDir::Files|QDir::NoDotAndDotDot, QDir::Name );
	QRegExp rx2 ( strL[1] );
	rx2.setPatternSyntax ( QRegExp::Wildcard );
	ifiles = ifiles.filter ( rx2 );
	if ( ifiles.count() == 0 )
	{
		QMessageBox::information ( this, "Burst Script", tr ( "No files to split!" ) );
		return;
	}
	//QStringList ofiles;ofiles.clear();
	QString dir_pat_main, dir_pat,  file_pat_main = strL[2], file_pat_temp;
	strL[2].replace ( QRegExp ( "^\\/" ), "" );
	if ( strL[2].contains ( "/" ) )
	{
		if ( strL[2].count ( "/" ) > 1 )
		{
			QMessageBox::warning ( this, tr ( "Invalid script" ), tr ( "Only subdirectories are supported (no subsubdirectories etc.). So output file pattern should contain no more than one \"/\"." ) );
			return;
		}
		dir_pat_main = strL[2].split ( "/" ).first();
		file_pat_main = strL[2].split ( "/" ).last();
	}
	int zeros = strL[3].toInt();
	if ( zeros <=0 ) zeros = 3;
	QString pass;
	if ( strL.count() > 4 )
	{
		pass = strL[4].replace(QRegExp("^\""), "").replace(QRegExp("\"$"), "");
		qDebug()<<"pass="<<pass;
	}

	if ( ifiles.count() > 1 )
	{
		prog_com = new QProgressDialog ( tr ( "Converting files" ), tr ( "Cancel" ), 0, ifiles.count() , this );
		prog_com->show();
		prog_com->setWindowModality ( Qt::WindowModal );
	}

	for ( int i = 0;i<ifiles.count();i++ )
	{
		if ( ifiles.count() > 1 )
		{
			if ( prog_com->wasCanceled() ) break;
			prog_com->setLabelText ( ifiles[i] );
		}
		ifiles[i] = lastdir+"/"+ifiles[i];
		qDebug() <<ifiles[i];
		file_pat_temp = file_pat_main;
		file_pat_temp.replace ( "*", QFileInfo ( ifiles[i] ).baseName() );
		QRegExp rx ( "(\\.|\\_|\\-|\\+|\\s|[A-Z]|[a-z]|[0-9])+" );
		if(!rx.exactMatch(file_pat_temp))
		{
			if(QMessageBox::question(this, "Burst Script", tr("Error while processing<br><b>%1<b><br>Due to pdftk limitations, prefix should contain only english symbols, digits, symbols \"-\", \"+\", \"_\" and spaces. Do you want to change prefix?<br>If you select \"No\", prefix will be set to burst_%2 ").arg(ifiles[i]).arg(i+1), QMessageBox::No|QMessageBox::Yes) == QMessageBox::Yes)
			{
				bool cont = false;
				while(!cont)
				file_pat_temp = QInputDialog::getText(this, "Burst Script", tr("Prefix:"));
				if(file_pat_temp.isEmpty())
				{
					file_pat_temp = tr("burst_%1").arg(i+1);
					QMessageBox::information(this, "Burst Script", tr("Prefix is set to %1").arg(file_pat_temp));
				}
				if(rx.exactMatch(file_pat_temp)) cont = true;
				else QMessageBox::warning(this, "Burst Script", tr("Unfortunately, this prefix is also unacceptable. Please, choose another one"));
			}
			else file_pat_temp = tr("burst_%1").arg(i+1);
		}
		if ( !dir_pat_main.isEmpty() )
		{
			dir_pat = dir_pat_main;
			dir_pat.replace ( "*", QFileInfo ( ifiles[i] ).baseName() );
			dir_pat = lastdir+"/"+dir_pat;
			qDebug() <<dir_pat;
			bool cont = true;
			if ( !QDir ( dir_pat ).exists() )
			{
				if ( !QDir().mkpath ( dir_pat ) )
				{
					QMessageBox::warning ( this, "Burst Script", tr ( "Failed to create directory.<br>%1 will be skipped" ).arg ( ifiles[i] ) );
					cont = false;
				}
			}
			if ( cont )
			{
				if ( !QFile ( dir_pat+"/"+QFileInfo ( ifiles[i] ).fileName() ).exists() )
				{
					if ( !QFile::copy ( ifiles[i], dir_pat+"/"+QFileInfo ( ifiles[i] ).fileName() ) )
						QMessageBox::warning ( this, "Burst Script", tr ( "Failed to copy file.<br>%1 will be skipped" ).arg ( ifiles[i] ) );
					else	onBurstCore ( dir_pat+"/"+QFileInfo ( ifiles[i] ).fileName(), file_pat_temp, zeros, pass );
				}
				else
				{
					if ( QMessageBox::question ( this, "Burst Script", tr ( "Directory %1<br>exists! It is possible that this script has been already executed in selected active directory. Some files may be overwritten! Continue anyway?" ).arg ( dir_pat ), QMessageBox::Yes|QMessageBox::No ) == QMessageBox::Yes )
						onBurstCore ( dir_pat+"/"+QFileInfo ( ifiles[i] ).fileName(), file_pat_temp, zeros, pass );
				}
			}
		}
		else onBurstCore ( lastdir+"/"+QFileInfo ( ifiles[i] ).fileName(), file_pat_temp, zeros, pass );
		if ( ifiles.count() > 1 )	prog_com->setValue ( i+1 );
	}
	if ( ifiles.count() > 1 ) prog_com->setValue ( ifiles.count() );
}

void SuslicTk::onTxtScript ( QString script )
{
	qDebug() <<"onTxtScript";
	QStringList strL = script.split ( " ", QString::SkipEmptyParts );
	if ( strL.count() < 4 )
	{
		qDebug() <<"invalid txt2pdf script";
		return;
	}
	QStringList ifiles = QDir ( lastdir ).entryList ( QStringList() <<"*.txt"<<"*.TXT", QDir::Files|QDir::NoDotAndDotDot, QDir::Name );
	QRegExp rx2 ( strL[1] );
	rx2.setPatternSyntax ( QRegExp::Wildcard );
	ifiles = ifiles.filter ( rx2 );
	if ( ifiles.count() == 0 )
	{
		QMessageBox::information ( this, "Txt2Pdf Script", tr ( "No files to convert!" ) );
		return;
	}
	QString ofile;strL[2].replace ( QRegExp ( "^\\/" ), "" );
	if ( ifiles.count() > 1 )
	{
		prog_com = new QProgressDialog ( tr ( "Converting files" ), tr ( "Cancel" ), 0, ifiles.count() , this );
		prog_com->show();
		prog_com->setWindowModality ( Qt::WindowModal );
	}

	for ( int i = 0;i<ifiles.count();i++ )
	{
		if ( ifiles.count() > 1 )
		{
			if ( prog_com->wasCanceled() ) break;
			prog_com->setLabelText ( ifiles[i] );
		}
		ifiles[i] = lastdir+"/"+ifiles[i];
		qDebug() <<ifiles[i];
		ofile = lastdir+"/"+strL[2];
		ofile.replace ( "*", QFileInfo ( ifiles[i] ).baseName() );
		if ( !ofile.endsWith ( ".pdf", Qt::CaseInsensitive ) ) ofile = ofile+".pdf";
		qDebug() << ofile;
		if ( ifiles[i] != ofile )
		{
			if ( QFile ( ofile ).exists() )
			{
				if ( QMessageBox::question ( this, "Txt2Pdf Script", tr ( "File<br>%1<br>exists. Overwrite?" ).arg ( ofile ), QMessageBox::Yes|QMessageBox::No ) == QMessageBox::No )
				{
					ofile = QFileDialog::getSaveFileName ( this, tr ( "Save as" ), QFileInfo ( ofile ).absolutePath() );
				}
			}
			if ( !ofile.isEmpty() )
			{
				if ( !QFileInfo ( ofile ).absoluteDir().exists() )
				{
					if ( !QDir().mkpath ( QFileInfo ( ofile ).absolutePath() ) )
						QMessageBox::warning ( this, "Txt2Pdf Script", tr ( "Failed to create directory.<br>%1 will be skipped" ).arg ( ifiles[i] ) );
					else onTxt2PdfCore ( ifiles[i], ofile, strL[3] );
				}
				else onTxt2PdfCore ( ifiles[i], ofile, strL[3] );
			}
		}
		else	addTextTE ( tr ( "<font color=red>Cannot convert %1 to %2. Input and output names should differ!</font>" ).arg ( ifiles[i] ).arg ( ofile ) );
		if ( ifiles.count() > 1 ) prog_com->setValue ( i+1 );
	}
	if ( ifiles.count() > 1 ) prog_com->setValue ( ifiles.count() );
}
